<?php
require("LdapAttribute.php");

class LdapEntry {
    var $dn;
    var $attrs;
    
    function LdapEntry($dn) {
        $this->dn=$dn;
        $this->attrs=array();
    }
    
    function selectLang($lang) {
        $listAttr=array_keys($this->attrs);
        foreach ($listAttr as $nameAttr) {
        	$attr=$this->attrs[$nameAttr];
        	$attr->setLanguage($lang);
        	$this->attrs[$nameAttr]=$attr;
        }
    }
    
    function addAttribute($attr,$value) {
        $nameAttr=$attr;
        $langAttr='';
        if (!(strpos($nameAttr,";lang-")===false)) {
        	$nameAttr=substr($attr,0,strpos($attr,";"));
        	$langAttr=substr($attr,strpos($attr,";")+6);
        }
        if (!array_key_exists($nameAttr,$this->attrs)) {
            $this->attrs[$nameAttr]=new LdapAttribute($nameAttr);
        }
        $this->attrs[$nameAttr]->addValue($value,$langAttr);
    }
    
    function getAttributes() {
        return $this->attrs;
    }

	function getObjectClassList() {
		return $this->attrs["objectClass"];
	}
	
	function getAttribute($attr) {
		$nameAttrs=array_keys($this->attrs);
		foreach ($nameAttrs as $nameAttr) {
			if (strcmp($nameAttr,$attr)==0) {
				return $this->attrs[$nameAttr];
			}
		}
		return "";
	}
    
	function getSingleAttribute($attr) {
		$nameAttrs=array_keys($this->attrs);
		foreach ($nameAttrs as $nameAttr) {
			if (strcmp($nameAttr,$attr)==0) {
				return $this->attrs[$nameAttr]->getFirstValue();
			}
		}
		return "";
	}
    
	function getAttributeValues() {
		$res=array();
		$nameAttrs=array_keys($this->attrs);
		foreach ($nameAttrs as $nameAttr) {
			if (strcmp($nameAttr,"objectClass")!=0) {
				$res[$nameAttr]=$this->attrs[$nameAttr]->getValues();
			}
		}
		return $res;
	}
    
    function getDN() {
        return $this->dn;
    }
    
    function getRDN() {
    	if (strpos($this->dn,",")===false) {
    		return $this->dn;
    	}
    	else {
    		return substr($this->dn,0,strpos($this->dn,","));
    	}
    }
    
    function _isFullAscii( $str ) {
		for( $i=0; $i<strlen($str); $i++ )
			if( ord( $str{$i} ) < 32 || ord( $str{$i} ) > 127 )
				return false;
		return true;
	}
 
    function _parseLDIFAttribute($name,$value) {
        if (!$this->_isFullAscii($value)) {
            return $name.":: ".base64_encode($value);
        }
        else {
            return $name.": ".$value;
        }
    }
    
    function getLDIF() {
        $res = 'dn: '.$this->dn."\n";
        $listAttr=array_keys($this->attrs);
        foreach ($listAttr as $nameAttr) {
            if (strcmp($nameAttr,"objectClass")==0) {
                $valuesAttr=$this->attrs[$nameAttr];
                foreach ($valuesAttr as $valueAttr) {
                    $res.=$this->_parseLDIFAttribute("objectClass",$valueAttr)."\n";
                }
            }
        }
        foreach ($listAttr as $nameAttr) {
            if (strcmp($nameAttr,"objectClass")!=0) {
                $valuesAttr=$this->attrs[$nameAttr];
                foreach ($valuesAttr as $valueAttr) {
                    $res.=$this->_parseLDIFAttribute($nameAttr,$valueAttr)."\n";
                }
            }
        }
        return $res;
    }
    
    function getXML() {
        $res = '<Entry dn="'.$this->dn.'">'."\n";
        $listAttr=array_keys($this->attrs);
        foreach ($listAttr as $nameAttr) {
            $nameAttr=ereg_replace("<","&lt;",$nameAttr);
            $nameAttr=ereg_replace("&","&amp;",$nameAttr);
            $nameAttr=ereg_replace(">","&gt;",$nameAttr);
            $nameAttr=ereg_replace('"',"&quot;",$nameAttr);
            $nameAttr=ereg_replace("'","&apos;",$nameAttr);
            $res.= '  <Attribute name="'.$nameAttr.'">'."\n";
            $attrEntry=$this->attrs[$nameAttr];
            foreach ($attrEntry->getArrayValues() as $value) {
            	$valueAttr=$value['value'];
            	if (!(strpos($nameAttr,";binary")===false)) {
            		$valueAttr=base64_encode($valueAttr);
            	}
                $valueAttr=ereg_replace("&","&amp;",$valueAttr); 
                $valueAttr=ereg_replace("<","&lt;",$valueAttr);
                $valueAttr=ereg_replace(">","&gt;",$valueAttr);
                $valueAttr=ereg_replace('"',"&quot;",$valueAttr);
                $valueAttr=ereg_replace("'","&apos;",$valueAttr);
                $res.= '    <AttributeValue ';
                if (array_key_exists("lang",$value)) {
                	$res.= 'lang="'.$value['lang'].'" ';
                }
                $res.= 'value="'.$valueAttr.'"/>'."\n";
            }
            $res.= '  </Attribute>'."\n";
        }
        $res.= '</Entry>'."\n";
        
        return $res;
    }
}
?>